package klesk.wi.weblog;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Locale;

/**
 * Converts an access web log (in ASCII format) to a comma separated
 * file (for further analysis).
 * 
 * @author Przemyslaw Klesk (<a href="mailto:pklesk@wi.zut.edu.pl">pklesk@wi.zut.edu.pl</a>)
 */
public class AccessLogConverter {
	
	public static void toCommaSeparatedFile(String aSourceFile, String aDestinationFile, long aMaximumLines, boolean aVerbose) throws Exception {		
		BufferedReader br = new BufferedReader(new FileReader(new File(aSourceFile)));
		BufferedWriter bw = new BufferedWriter(new FileWriter(new File(aDestinationFile)));
		SimpleDateFormat sdf = new SimpleDateFormat("dd/MMM/yyyy:HH:mm:ss", Locale.ENGLISH);		
		
		int k = 1;
		while (true) {
			String lineIn = br.readLine();
			if (lineIn == null) {//end of file reached
				br.close();
				bw.close();
				break;
			}
			if (aVerbose) System.out.println(k + " in: " + lineIn);
			
			
			String host = "";
			try {
				host = lineIn.substring(0, lineIn.indexOf("-")).trim();				
			}
			catch (Exception e) {
				host = "?";
			};
			
			String timeStamp = "?"; 
			try{
				timeStamp = lineIn.substring(lineIn.indexOf("[") + 1, lineIn.indexOf("]"));
				timeStamp = timeStamp.substring(0, timeStamp.indexOf("-")).trim();
				Date date = sdf.parse(timeStamp);
				timeStamp = Long.toString(date.getTime()); //time in miliseconds
			}
			catch (Exception e) {
				timeStamp = "?";
			};
			
			String request = "";
			try {
				request = lineIn.substring(lineIn.indexOf("\"") + 1, lineIn.lastIndexOf("\""));
				request = request.replaceAll(",", "|");
			}
			catch (Exception e) {
				request = "?";
			}
			
						
			String method = ""; 
			try {	
				method = request.substring(0, request.indexOf(" "));
			} catch (Exception e) {
				method = "?";
			}
			
			String object = "";
			try {
				object = request.substring(request.indexOf(" ") + 1, Math.max(request.indexOf(" HTTP"), request.length()));
			} catch (Exception e) {
				object = "?";
			}
			
			String lastPart = "";
			try {
				lastPart = lineIn.substring(lineIn.lastIndexOf("\"") + 2);
			} catch (Exception e) {
				lastPart = "?";
			}
			
			String responseCode = "";
			try {
				responseCode = lastPart.substring(0, lastPart.indexOf(" "));
			} catch (Exception e) {
				responseCode = "?";
			}
			
			String size = "";
			try {
				size = lastPart.substring(lastPart.indexOf(" ") + 1);
				if (size.equals("-")) size = "0";
			} catch (Exception e) {
				size = "?";
			}
			
			String lineOut = host + "," + timeStamp + "," + method + "," 
				+ object + "," + responseCode + "," + size + "\n";
			 
			if (aVerbose) System.out.print(k + " out: " + lineOut);
			bw.write(lineOut);
			
			if (!aVerbose) {
				if (k % Math.round(aMaximumLines * 0.1) == 0) 
					System.out.println("Progress: " + Math.round(100 * k / aMaximumLines) + "%.");
			}
			
			if (++k > aMaximumLines) {
				br.close();
				bw.close();
				break;
			}
		}
	}	
	
	public static void main(String[] aArgs) throws Exception {
		System.out.println("Starting...");
		
		AccessLogConverter.toCommaSeparatedFile("c:/UofS_access_log", "c:/out.txt", 10, true);
		//AccessLogConverter.toCommaSeparatedFile("c:/UofS_access_log", "c:/out.txt", Double.valueOf(Double.POSITIVE_INFINITY).longValue(), false);		
		
		System.out.println("Done.");
	}		
}